@@ -43,6 +43,8 @@ gls <-
     glsSt <-
         glsStruct(corStruct = correlation, varStruct = varFunc(weights))
 
+    ## we need to resolve '.' in the formula here
+    model <- terms(model, data=data)
     ## extract a data frame with enough information to evaluate
     ## formula, groups, corStruct, and varStruct
     mfArgs <- list(formula = asOneFormula(formula(glsSt), model, groups),
@@ -78,7 +80,7 @@ gls <-
     N <- nrow(X)
     p <- ncol(X)				# number of coefficients
     parAssign <- attr(X, "assign")
-    fTerms <- terms(as.formula(model))
+    fTerms <- terms(as.formula(model), data=data)
     namTerms <- attr(fTerms, "term.labels")
     if (attr(fTerms, "intercept") > 0) {
         namTerms <- c("(Intercept)", namTerms)
@@ -103,7 +105,7 @@ gls <-
     repeat {
         oldPars <- c(attr(glsSt, "glsFit")[["beta"]], coef(glsSt))
         if (length(coef(glsSt))) {		# needs ms()
-            optRes <- if (exists("nlminb", mode = "function")) {
+            optRes <- if (controlvals$opt == "nlminb") {
                 nlminb(c(coef(glsSt)),
                        function(glsPars) -logLik(glsSt, glsPars),
                        control = list(trace = controlvals$msVerbose,
@@ -1295,7 +1297,8 @@ glsControl <-
              msScale = lmeScale, msVerbose = FALSE, singular.ok = FALSE,
              qrTol = .Machine$single.eps, returnObject = FALSE,
              apVar = TRUE, .relStep = (.Machine$double.eps)^(1/3),
-             nlmStepMax = 100.0, optimMethod = "BFGS",
+             nlmStepMax = 100.0,
+	     opt = c("nlminb", "optim"),  optimMethod = "BFGS",
              minAbsParApVar = 0.05, natural = TRUE)
 {
     list(maxIter = maxIter, msMaxIter = msMaxIter, tolerance = tolerance,
@@ -1303,8 +1306,8 @@ glsControl <-
          singular.ok = singular.ok, qrTol = qrTol,
          returnObject = returnObject, apVar = apVar,
          minAbsParApVar = minAbsParApVar, .relStep = .relStep,
-         nlmStepMax = nlmStepMax, optimMethod,
-         natural = natural)
+         nlmStepMax = nlmStepMax, opt = match.arg(opt), 
+	 optimMethod = optimMethod, natural = natural)
 }
 
 ### local generics for objects inheriting from class lme
